local L = AceLibrary("AceLocale-2.2"):new("OptiTaunt")
OptiTaunt = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

--local declarations
local tauntsoundfile = "Interface\\AddOns\\OptiTaunt\\Sounds\\combo.ogg"

-- setup options
OptiTaunt.options = { 
		type='group',
		args = {
			raid = {
				type = 'toggle',
				name = L["opt_name_raid"],
				desc = L["opt_desc_raid"],
				get = "IsShowInRaidChat",
				set = "ToggleShowInRaidChat",
			},
			ctra = {
				type = 'toggle',
				name = L["opt_name_ctra"],
				desc = L["opt_desc_ctra"],
				get = "IsShowInCtra",
				set = "ToggleShowInCtra",
			},
			party = {
				type = 'toggle',
				name = L["opt_name_party"],
				desc = L["opt_desc_party"],
				get = "IsShowInPartyChat",
				set = "ToggleShowInPartyChat",
			},
			autowhisper = {
				type = 'toggle',
				name = L["opt_name_autowhipser"],
				desc = L["opt_desc_autowhisper"],
				get = "IsAutoWhisper",
				set = "ToggleAutoWhisper",
			},
			audio = {
				type = 'toggle',
				name = L["opt_name_audio"],
				desc = L["opt_desc_audio"],
				get = "IsGiveAudioWarning",
				set = "ToggleGiveAudioWarning",
			},
		},
	}

--setup FuBar-2.0
OptiTaunt.hasIcon = true
OptiTaunt.defaultMinimapPosition = 340
OptiTaunt.cannotDetachTooltip = true
OptiTaunt.independentProfile = true
OptiTaunt.defaultPosition = "LEFT"
OptiTaunt.hideWithoutStandby = true
OptiTaunt.hasNoColor = true

local tablet = AceLibrary("Tablet-2.0")

function OptiTaunt:OnTooltipUpdate()
	tablet:SetHint(L["Tooltip"])
end

function OptiTaunt:OnInitialize()
	self:RegisterDB("OptiTauntDB", "OptiTauntDBPC")
	self:RegisterDefaults("profile", {
		showInRaidChat = true,
		showInCtra = false,
		showInPartyChat = true,
		showInWhisperChat = true,
		giveAudioWarning = true,
		autoWhisper = true,
	} )
	
	self:RegisterChatCommand(L["Slash-Commands"], self.options)
	self.opt = self.db.profile
	self.OnMenuRequest = self.options
end

function OptiTaunt:OnProfileEnable()
	self.opt = self.db.profile
end

function OptiTaunt:OnEnable()
    self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
	--this is for debugging
	--self:RegisterEvent("CHAT_MSG_SAY")
end

function OptiTaunt:IsShowInRaidChat()
    return self.db.profile.showInRaidChat
end

function OptiTaunt:ToggleShowInRaidChat()
    self.db.profile.showInRaidChat = not self.db.profile.showInRaidChat
end

function OptiTaunt:IsShowInCtra()
    return self.db.profile.showInCtra
end

function OptiTaunt:ToggleShowInCtra()
    self.db.profile.showInCtra = not self.db.profile.showInCtra
end

function OptiTaunt:IsShowInPartyChat()
    return self.db.profile.showInPartyChat
end

function OptiTaunt:ToggleShowInPartyChat()
    self.db.profile.showInPartyChat = not self.db.profile.showInPartyChat
end

function OptiTaunt:IsAutoWhisper()
    return self.db.profile.autoWhisper
end

function OptiTaunt:ToggleAutoWhisper()
    self.db.profile.autoWhisper = not self.db.profile.autoWhisper
end

function OptiTaunt:IsGiveAudioWarning()
    return self.db.profile.giveAudioWarning
end

function OptiTaunt:ToggleGiveAudioWarning()
    self.db.profile.giveAudioWarning = not self.db.profile.giveAudioWarning
end

--this is for debugging
--[[function OptiTaunt:CHAT_MSG_SAY(arg1)
	for ability,mob in string.gmatch(arg1, L["Combatlog-Resist-Message"]) do
		if (ability == L["Taunt"]) or (ability == L["Growl"]) or (ability == L["Mocking Blow"]) then
			
			self:Print("player= "..UnitName("player"))
			--self:Print("target= "..UnitName("target"))
			
			--if (UnitName("player") == UnitName("target")) then
			if not (UnitIsUnit("player","target")) then
					OptiTaunt:giveAlert(ability, mob, "resist")
			end

		end
	end
	for ability,mob in string.gmatch(arg1, L["Combatlog-Immune-Message"]) do
		if (ability == L["Taunt"]) or (ability == L["Growl"]) or (ability == L["Mocking Blow"]) then
			--if not (targettarget == player) and (UnitPlayerControlled("targettarget")) and not (UnitIsPlayer("targettarget")) then
				OptiTaunt:giveAlert(ability, mob, "immune")
			--end
		end
	end
end]]

function OptiTaunt:CHAT_MSG_SPELL_SELF_DAMAGE(arg1)
	for ability,mob in string.gmatch(arg1, L["Combatlog-Resist-Message"]) do
		if (ability == L["Taunt"]) or (ability == L["Growl"]) or (ability == L["Mocking Blow"]) then
			--debug stuff
			--[[self:Print("Debug... widerstanden!")
			if UnitName("target") then
				self:Print("Debug... Target: "..UnitName("target"))
			end
			if UnitName("targettarget") then
				self:Print("Debug... TargetTarget: "..UnitName("targettarget"))
			end]]
			if not (UnitIsUnit("player","targettarget")) and (UnitIsPlayer("targettarget")) and (UnitPlayerControlled("targettarget")) and (UnitIsFriend("targettarget")) then
					OptiTaunt:giveAlert(ability, mob, "resist")
			end
		end
	end
	
	for ability,mob in string.gmatch(arg1, L["Combatlog-Immune-Message"]) do
		if (ability == L["Taunt"]) or (ability == L["Growl"]) or (ability == L["Mocking Blow"]) then
			--debug stuff
			--[[self:Print("Debug... immun!")
			if UnitName("target") then
				self:Print("Debug... Target: "..UnitName("target"))
			end
			if UnitName("targettarget") then
				self:Print("Debug... TargetTarget: "..UnitName("targettarget"))
			end]]
			if not (UnitIsUnit("player","targettarget")) and (UnitIsPlayer("targettarget")) and (UnitPlayerControlled("targettarget")) and (UnitIsFriend("targettarget")) then
				OptiTaunt:giveAlert(ability, mob, "immune")
			end
		end
	end
end

function OptiTaunt:giveAlert(ability,mob,msgtype)
	if self.db.profile.showInCtra then
		if (GetNumRaidMembers() > 0) and (IsRaidOfficer() == 1) then
			if (msgtype == "resist") then
				if (IsAddOnLoaded("CT_RaidAssist")) then
					CT_RA_AddMessage("MS "..L[Warning].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Message-Part2"])
				elseif (IsAddOnLoaded("oRA2")) then
					oRALRaidWarn:SendRS(L[Warning].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Message-Part2"])
				end
			elseif (msgtype == "immune") then
				if (IsAddOnLoaded("CT_RaidAssist")) then
					CT_RA_AddMessage("MS "..L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!")
				elseif (IsAddOnLoaded("oRA2")) then
					oRALRaidWarn:SendRS(L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!");
				end
			end
		end
	end
	
	if self.db.profile.showInRaidChat then
		if (GetNumRaidMembers() > 0) then
			if (msgtype == "resist") then
				SendChatMessage(L[Warning].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Message-Part2"], "RAID");
			elseif (msgtype == "immune") then
				SendChatMessage(L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!", "RAID");
			end
		end
	end
	
	if self.db.profile.showInPartyChat then
		if (GetNumPartyMembers() > 0) then					
			if (msgtype == "resist") then
				SendChatMessage(L[Warning].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Message-Part2"], "PARTY");
			elseif (msgtype == "immune") then
				SendChatMessage(L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!", "PARTY");
			end
		end
	end
	
	if self.db.profile.autoWhisper and (UnitName("targettarget") ~= nil) and (UnitPlayerControlled("targettarget")) then
		if (msgtype == "resist") then
			SendChatMessage(L["Warning"].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Whisper-Message"], "WHISPER", GEMEINSPRACHE, UnitName("targettarget"));
		elseif (msgtype == "immune") then
			SendChatMessage(L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!", "WHISPER", GEMEINSPRACHE, UnitName("targettarget"));
		end
	end
	
	if self.db.profile.giveAudioWarning then
		PlaySoundFile(tauntsoundfile);
	end
	
	if (msgtype == "resist") then
		UIErrorsFrame:AddMessage(L["Warning"].." "..mob.." "..L["Resist-Message-Part1"].." "..ability.." "..L["Resist-Message-Part2"], 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
	elseif (msgtype == "immune") then
		UIErrorsFrame:AddMessage(L["Warning"].." "..mob.." "..L["Immune-Message-Part1"].." "..ability.."!", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
	end
end